@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.35 2003/01/31 09:27:02 bazsi Exp $ */
+ * $Id: io.c,v 1.37 2003/05/22 16:56:59 bazsi Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -373,9 +373,13 @@ static int do_recv(struct abstract_read **r, UINT32 length, UINT8 *buffer, abstr
 	for (;;) {
 		int res;
 		
+		/* This is another workaround in Linux kernel bugs */
+		
+		((struct sockaddr *) addr)->sa_family = 0;
+		
 		res = recvfrom(closure->fd, buffer, length, 0, (struct sockaddr *) addr, (socklen_t *) addrlen);
 
-		if (*addrlen == 2) {
+		if (*addrlen == 2 || (*addrlen >= 2 && ((struct sockaddr *) addr)->sa_family == 0)) {
 			/* HACK: this is a workaround of a Linux 2.2 & 2.4 bug,
 			 * it doesn't return anything sensible in sockaddr buf 
 			 */
@@ -1013,6 +1017,7 @@ struct address_info *sockaddr2address_info(size_t addr_len,
 		struct sockaddr_in *in = (struct sockaddr_in *) addr;
 		UINT32 ip, port;
 
+		assert(addr_len == sizeof(struct sockaddr_in));
 		ip = ntohl(in->sin_addr.s_addr);
 		port = ntohs(in->sin_port);
 		a = make_inet_address(c_format_cstring("%i.%i.%i.%i",
