@@ -18,7 +18,7 @@
  *
  * Inspired by nsyslog, originally written by Darren Reed.
  *
- * $Id: pkt_buffer.c,v 1.16 2003/01/20 15:30:25 bazsi Exp $
+ * $Id: pkt_buffer.c,v 1.17 2003/01/31 09:27:02 bazsi Exp $
  *
  ***************************************************************************/
  
@@ -32,12 +32,15 @@ struct buffer_node {
 	struct ol_string *packet;
 };
 
+int pktbuf_dropped_pkts = 0;
+
 /* CLASS:
      (class
        (name pkt_buffer)
        (super abstract_buffer)
        (vars
          (queue special-struct "struct ol_queue" #f do_free_buffer)
+	 (pkt_flush simple int)
 	 (queue_min simple int)
          (queue_size simple int)
          (queue_max simple int)))
@@ -70,6 +73,7 @@ static int do_write_str(struct abstract_write *c, struct ol_string *string)
 	}
 	if (self->queue_size == self->queue_max) {
 		/* fifo full */
+		pktbuf_dropped_pkts++;
 		ol_string_free(string);
 		return ST_FAIL | ST_OK;
 	}
@@ -85,7 +89,7 @@ static int do_write_str(struct abstract_write *c, struct ol_string *string)
 	return ST_OK | ST_GOON;
 }
 
-static int do_flush(struct abstract_buffer *c, struct abstract_write *w)
+static int do_flush_stream(struct abstract_buffer *c, struct abstract_write *w)
 {
 	CAST(pkt_buffer, self, c); 
 	char buffer[4096];
@@ -154,6 +158,53 @@ static int do_flush(struct abstract_buffer *c, struct abstract_write *w)
 	return ST_OK | ST_GOON;
 }
 
+static int do_flush_pkt(struct abstract_buffer *c, struct abstract_write *w)
+{
+ 	CAST(pkt_buffer, self, c);
+	int res;
+
+	FOR_QUEUE(&self->queue, struct buffer_node *, item) {
+
+		res = A_WRITE(w, item->packet->length, item->packet->data);
+		if (res >= 0) {
+                        if (item->packet->length == res) {
+                                self->queue_size--;
+                                ol_queue_remove((struct ol_queue_node *) item);
+                                ol_string_free(item->packet);
+                                ol_space_free(item);
+                                if (self->super.writable)
+                                        (*self->super.writable) = 1;
+			}
+                        else if (res != 0) {
+                                struct ol_string *s;
+                                s = item->packet;
+                                item->packet = c_format("%s", s->length - res, s->data + res);
+                                ol_string_free(s);
+                                break;
+			}
+                        else {
+                              	break;
+			}
+		}
+                else {
+                      	verbose("pkt_buffer::do_flush(): Error flushing data\n");
+			return ST_DIE;
+		}
+	}
+        return ST_OK | ST_GOON;
+}
+
+static int do_flush(struct abstract_buffer *c, struct abstract_write *w)
+{
+	CAST(pkt_buffer, self, c);
+
+	if (self->pkt_flush)
+		return do_flush_pkt(c, w);
+	else
+		return do_flush_stream(c, w);
+}
+
+
 static int do_prepare_write(struct abstract_buffer *c)
 {
 	CAST(pkt_buffer, self, c);
@@ -188,10 +239,11 @@ struct abstract_buffer *make_pkt_buffer(int queue_max)
 	return &self->super;
 }
 
-struct abstract_buffer *make_pkt_buffer_ext(int queue_min, int queue_max)
+struct abstract_buffer *make_pkt_buffer_ext(int queue_min, int queue_max, int pkt_flush)
 {
 	struct abstract_buffer *b = make_pkt_buffer(queue_max);
 	((struct pkt_buffer *) b)->queue_min = queue_min;
+	((struct pkt_buffer *) b)->pkt_flush = pkt_flush;
 
 	return b;
 }
